// Bibliotheken für LCD includen
#include <Wire.h> // I2C
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>

// Ein lcd Objekt zur Ansteuerung erstellen
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

// Diese #defines machen es einfach die Farben
// für die Hintergrundbeleuchtung im Code zu verwenden.
#define RED     0x1
#define YELLOW  0x3
#define GREEN   0x2
#define TEAL    0x6
#define BLUE    0x4
#define VIOLET  0x5
#define WHITE   0x7

//      Funktion  Pin  Beschreibung
#define DIR_A     12   // Richtung Motor A
#define DIR_B     13   // Richtung Motor B
#define PWM_A     3    // PWM Motor A
#define PWM_B     11   // PWM Motor B
#define BRAKE_A   9    // Bremse Motor A
#define BRAKE_B   8    // Bremse Motor B


#define FULL_SPEED_A      141  // Vollgas ist 255. 141 für Motor A, da dieser bei meinem Bot langsamer lief!
#define FULL_SPEED_B      128  // Vollgas ist 255. 128 für Motor B.
#define TURN_SPEED        128  // Drehgeschwindigkeit
// Ein paar Richtungen für die Fahr- und Drehfunktion
#define LINKS             LOW
#define RECHTS           HIGH
#define VORWAERTS        HIGH
#define RUECKWAERTS       LOW


// Vorwärts fahren
void fahreGeradeaus(boolean richtung)
{
  // Beide Motoren auf geradeaus stellen
  digitalWrite(DIR_A, richtung);
  digitalWrite(DIR_B, richtung);
  
  // Geschwindigkeit setzen
  analogWrite(PWM_A, FULL_SPEED_A);
  analogWrite(PWM_B, FULL_SPEED_B);
  
  // Losfahren!
  digitalWrite(BRAKE_A, LOW);
  digitalWrite(BRAKE_B, LOW);
}


// Lässt den Roboter nach links (LEFT) oder rechts (RIGHT) drehen
void drehen(boolean richtung)
{
  // Motor A in entgegen der "RICHTUNG" drehen
  digitalWrite(DIR_A, !richtung);

  // Motor B in die "RICHTUNG" drehen
  digitalWrite(DIR_B, richtung);
   
  // Geschwindigkeit für das Drehen einstellen
  analogWrite(PWM_A, TURN_SPEED);
  analogWrite(PWM_B, TURN_SPEED);
  
  // Bremsen lösen
  digitalWrite(BRAKE_A, LOW);
  digitalWrite(BRAKE_B, LOW);
}


void anhalten()
{
  // Bremsen
  digitalWrite(BRAKE_A, HIGH);
  digitalWrite(BRAKE_B, HIGH);
}


void setup()
{
  // Pins für Motorrichtungen als Ausgang definieren
  pinMode(DIR_A, OUTPUT);
  pinMode(DIR_B, OUTPUT);

  // Pins für Bremsen als Ausgang definieren
  pinMode(BRAKE_A, OUTPUT);
  pinMode(BRAKE_B, OUTPUT);
  
  // Bremsen anziehen!
  anhalten();
  
  // Das LCD ist 16 Zeichen breit und hat 2 Zeilen
  lcd.begin(16, 2);

  // Text komplett löschen
  lcd.clear();

  // Hintergrundfarbe auf weiß setzen
  lcd.setBacklight(WHITE);

  // Text ausgeben
  //         1234567890123456
  lcd.print("  Ich warte...  ");
}


void loop()
{
  byte buttons = 0;


  // Jetzt die Tasten des LCD auslesen
  buttons = lcd.readButtons();

  if (buttons != 0)
  {
    // Text komplett löschen
    lcd.clear();
    
    // Cursor auf 0,0 setzen
    lcd.setCursor(0,0);
    //         1234567890123456
    lcd.print("Fahrtrichtung:  ");

    if (buttons == BUTTON_UP)
    {
      // Text ausgeben
      lcd.setCursor(0,1);
      //         1234567890123456
      lcd.print("Links...........");

      // Hintergrundbeleuchtung/farbe ändern
      lcd.setBacklight(RED);

      // Roboter links(!) herum drehen
      drehen(LINKS);
    }
    
    if (buttons == BUTTON_DOWN)
    {
      // Text ausgeben
      lcd.setCursor(0,1);
      //         1234567890123456
      lcd.print("Rechts..........");

      // Hintergrundbeleuchtung/farbe ändern
      lcd.setBacklight(YELLOW);

      // Roboter rechts(!) herum drehen
      drehen(RECHTS);
    }
    
    if (buttons == BUTTON_LEFT)
    {
      // Text ausgeben
      lcd.setCursor(0,1);
      //         1234567890123456
      lcd.print("Rueckwaerts.....");

      // Hintergrundbeleuchtung/farbe ändern
      lcd.setBacklight(GREEN);

      // Roboter rückwärts(!) fahren
      fahreGeradeaus(RUECKWAERTS);
    }
    
    if (buttons == BUTTON_RIGHT)
    {
      // Text ausgeben
      lcd.setCursor(0,1);
      //         1234567890123456
      lcd.print("Vorwaerts.......");

      // Hintergrundbeleuchtung/farbe ändern
      lcd.setBacklight(TEAL);

      // Roboter vorwärts(!) fahren
      fahreGeradeaus(VORWAERTS);
    }
  }
  else
  {
    // KEINE Taste gedrückt

    // Text komplett löschen
    lcd.clear();
  
    // Hintergrundfarbe auf weiß setzen
    lcd.setBacklight(WHITE);
    
    // Roboter anhalten
    anhalten();
  }
}

